#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# Trashman v1.5.0
# A Python trash manager.
# Copyright (C) 2011–2015, Chris Warrick.
# See /LICENSE for licensing information.

"""
    trashman.backends.dummytrash
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    A dummy backend, printing all the requests it gets.

    :Copyright: © 2011–2015, Chris Warrick.
    :License: BSD (see /LICENSE).
"""
import logging


### DummyTrash     A dummy trash           ###
class DummyTrash(object):
    """
    A dummy trash, printing and logging actions
    that would be performed.
    """

    trashdir = None
    filedir = None
    infodir = None
    logger = logging.getLogger('DummyTrash')
    logquiet = False

    def log(self, msg, lvl='debug'):
        """Log a message."""
        if not self.logquiet:
            print('DummyTrash: {}'.format(msg))

        if lvl != 'debug':
            raise NotImplementedError

        self.logger.debug(msg)

    def regenerate(self):
        """Regenerate the trash and recreate metadata."""
        self.log('regenerating')

    def empty(self, verbose):
        """Empty the trash."""
        self.log('emptying (verbose={})'.format(verbose))
        self.regenerate()

    def list(self, human=True):
        """List the trash contents."""
        if human:
            self.log('listing contents (on stdout; human=True)')
        else:
            self.log('listing contents (return; human=False)')

    def trash(self, filepath, verbose):
        """Move specified file to trash."""
        self.log('trashing file {} (verbose={})'.format(filepath, verbose))

    def restore(self, filename, verbose):
        """Restore a file from trash."""
        self.log('restoring file {} (verbose={})'.format(filename, verbose))



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# Trashman v1.5.0
# A Python trash manager.
# Copyright (C) 2011–2015, Chris Warrick.
# See /LICENSE for licensing information.

"""
    trashman.main()
    ~~~~~~~~~~~~~~~

    Trashman’s main routine.

    :Copyright: © 2011–2015, Chris Warrick.
    :License: BSD (see /LICENSE).
"""

from . import DS, _, TMError, __version__, __pyver__
from .backends import select
import argparse


def main():
    """The main routine."""
    verstring = 'Trashman v'+__version__
    # TRANSLATORS: translate the whole sentence.
    # Alternatively, use translation instead of locale.
    locale = _('LANG locale by AUTHOR <MAIL@IF.YOU.WANT>')
    if locale != 'LANG locale by AUTHOR <MAIL@IF.YOU.WANT>':
        verstring = ' | '.join([verstring, locale])
    DS.log.info('Initialized, parsing arguments.')
    parser = argparse.ArgumentParser(description=_('Trashman -- a Python'
                    ' trash manager.'),
            formatter_class=argparse.RawTextHelpFormatter)

    parser.add_argument('-V', '--version', action='version',
                version=verstring,
                help=_('show version number and quit'))

    argopt = parser.add_argument_group(_('options'))
    argopr = parser.add_argument_group(_('operations'))

    argopt.add_argument('-v', '--verbose', action='store_true', default=False,
                        dest='verbose', help=_('explain what is being done'))
    argopt.add_argument('-b', '--backend', action='store', default='config',
                        dest='backend', help=_('select the backend (default: '
                                               'config->auto->xdg)'))

    argopr.add_argument('-e', '--empty', action='store_true', default=False,
                        dest='empty', help=_('empty the trash and quit'))
    argopr.add_argument('-l', '--list', action='store_true', default=False,
                        dest='flist', help=_('list the files in trash and '
                                             'quit'))
    argopr.add_argument('-r', '--restore', action='store_true', default=False,
                        dest='restore', help=_('restore FILE(s) from trash'))
    argopr.add_argument('-w', '--trash-location', action='store_true',
                        default=False, dest='showtloc', help=_('print the '
                        'trash location and quit'))
    argopr.add_argument('-W', '--files-location', action='store_true',
                        default=False, dest='showfloc', help=_('print the '
                        'trashed files location and quit'))
    parser.add_argument('files', metavar=_('FILE'), action='store', nargs='*',
                        help=_('files to trash'))
    args = parser.parse_args()

    quit_notrash = False

    select(args.backend)
    if args.backend == 'info':
        quit_notrash = True
    else:
        DS.trash.regenerate()

    if args.flist:
        DS.trash.list()
        quit_notrash = True

    if args.showtloc:
        print(DS.trash.trashdir)
        quit_notrash = True

    if args.showfloc:
        print(DS.trash.filedir)
        quit_notrash = True

    if args.restore:
        for fileres in args.files:
            try:
                DS.trash.restore(fileres, args.verbose)
            except TMError as e:
                print(_('ERROR:') + ' ' + e.msg)
                if e.src == 'restore' and e.info == 'nodir':
                    if __pyver__[0] == 3:
                        i = input('Restore to current working directory? '
                                  '[Y/n] ')
                    else:

                        i = raw_input('Restore to current working directory? '
                                      '[Y/n] ')
                    if not i.lower().startswith('n'):
                        DS.trash.restore(fileres, args.verbose, tocwd=True)
        quit_notrash = True

    if args.empty:
        DS.trash.empty(args.verbose)
        quit_notrash = True

    if quit_notrash:
        exit()
    else:
        # Did not quit?  We can trash.
        for f in args.files:
            try:
                DS.trash.trash(f, args.verbose)
            except OSError as e:
                print(_('ERROR:') + ' ' + e.strerror + ' ({})'.format(f))

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# Trashman v1.5.0
# A Python trash manager.
# Copyright (C) 2011–2015, Chris Warrick.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the author of this software nor the names of
#    contributors to this software may be used to endorse or promote
#    products derived from this software without specific prior written
#    consent.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
    trashman
    ~~~~~~~~

    A Python trash manager.

    :Copyright: © 2011–2015, Chris Warrick.
    :License: BSD (see /LICENSE).
"""

__title__ = 'Trashman'
__version__ = '1.5.0'
__author__ = 'Chris Warrick'
__license__ = '3-clause BSD'
__docformat__ = 'restructuredtext en'

import sys
import os
import gettext

__pyver__ = sys.version_info

G = gettext.translation('trashman', '/usr/share/locale', fallback='C')
_ = G.gettext


def size_dir(sdir):
    """Get the size of a directory.  Based on code found online."""
    size = os.path.getsize(sdir)

    for item in os.listdir(sdir):
        item = os.path.join(sdir, item)

        if os.path.isfile(item):
            size = size + os.path.getsize(item)
        elif os.path.isdir(item):
            size = size + size_dir(item)

    return size


### TMError         errors raised here      ###
class TMError(Exception):
    """Exceptions raised by the Trashman."""

    def __init__(self, src, info, msg):
        """TMError init."""
        DS.log.error('(auto TMError       ) [{}/{}]'.format(src, info) + msg)
        self.src = src
        self.info = info
        self.msg = msg

    def __str__(self):
        """You want to see error messages, don’t you?"""
        return self.msg

from .tmds import TMDS
DS = TMDS()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# Trashman v1.5.0
# A Python trash manager.
# Copyright (C) 2011–2015, Chris Warrick.
# See /LICENSE for licensing information.

"""
    trashman.backends.xdgtrash
    ~~~~~~~~~~~~~~~~~~~~~~~~~~

    An XDG trash implementation.

    :Copyright: © 2011–2015, Chris Warrick.
    :License: BSD (see /LICENSE).
"""

from .. import DS, _, TMError, size_dir
from .trash import Trash
import shutil
import os
import os.path
import datetime
import sys
import logging

try:
    import configparser
except ImportError:
    import ConfigParser as configparser


### XDGTrash       XDG trash backend       ###
class XDGTrash(Trash):
    """XDG trash backend."""

    if os.getenv('XDG_DATA_HOME') is None:
        trashdir = os.path.expanduser('~/.local/share/Trash')
    else:
        trashdir = os.getenv('XDG_DATA_HOME') + '/Trash'

    try:
        if not os.path.exists(trashdir):
            os.mkdir(trashdir)
    except OSError:
        DS.error('Couldn’t access the proper directory, temporary trash \
is in in /tmp/TRASH')
        trashdir = os.path.join('tmp' 'TRASH')

    filedir = trashdir + '/files/'
    infodir = trashdir + '/info/'
    logger = logging.getLogger('XDGTrash')

    def regenerate(self):
        """Regenerate the trash and recreate metadata."""
        self.log('regenerating')
        zerosize = False

        if not os.path.exists(self.trashdir):
            os.mkdir(self.trashdir)
            zerosize = True

        if ((not os.path.exists(self.filedir)) or
                (not os.path.exists(self.infodir))):
            os.mkdir(self.filedir)
            os.mkdir(self.infodir)
            zerosize = True
        if not zerosize:
            trashsize = (size_dir(self.filedir) + size_dir(self.infodir))
        else:
            trashsize = 0

        infofile = '[Cached]\nSize=' + str(trashsize) + '\n'
        fh = open(os.path.join(self.trashdir, 'metadata'), 'w')
        fh.write(infofile)
        fh.close()

    def empty(self, verbose):
        """Empty the trash."""
        self.log('emptying (verbose={})'.format(verbose))
        shutil.rmtree(self.filedir)
        shutil.rmtree(self.infodir)
        self.regenerate()
        if verbose:
            sys.stderr.write(_('emptied the trash\n'))

    def list(self, human=True):
        """List the trash contents."""
        if human:
            self.log('listing contents (on stdout; human=True)')
        else:
            self.log('listing contents (return; human=False)')
        dirs = []
        files = []
        for f in os.listdir(self.filedir):
            if os.path.isdir(self.filedir + f):
                dirs.append(f)
            else:
                files.append(f)

        dirs.sort()
        files.sort()

        allfiles = []

        for i in dirs:
            allfiles.append(i + '/')

        for i in files:
            allfiles.append(i)

        if human:
            if allfiles != []:
                print('\n'.join(allfiles))
        else:
            return allfiles

    def trash(self, filepath, verbose):
        """Move specified file to trash."""
        self.log('trashing file {} (verbose={})'.format(filepath, verbose))
        # Filename alteration, a big mess.
        filename = os.path.basename(filepath)
        fileext = os.path.splitext(filename)

        tomove = filename
        collision = True
        i = 1

        while collision:
            if os.path.lexists(self.filedir + tomove):
                tomove = fileext[0] + ' ' + str(i) + fileext[1]
                i = i + 1
            else:
                collision = False

        infofile = """[Trash Info]
Path={}
DeletionDate={}
""".format(os.path.realpath(filepath),
           datetime.datetime.now().strftime('%Y-%m-%dT%H:%m:%S'))

        os.rename(filepath, self.filedir + tomove)

        f = open(os.path.join(self.infodir, tomove + '.trashinfo'), 'w')
        f.write(infofile)
        f.close()

        self.regenerate()

        if verbose:
            sys.stderr.write(_('trashed \'{}\'\n').format(filename))

    def restore(self, filename, verbose, tocwd=False):
        """Restore a file from trash."""
        self.log('restoring file {} (verbose={}, tocwd={})'.format(filename,
                 verbose, tocwd))
        info = configparser.ConfigParser()
        if os.path.exists(os.path.join(self.filedir, filename)):
            info.read(os.path.join(self.infodir, filename + '.trashinfo'))
            restname = os.path.basename(info.get('Trash Info', 'Path'))

            if tocwd:
                restdir = os.path.abspath('.')
            else:
                restdir = os.path.dirname(info.get('Trash Info', 'Path'))

            restfile = os.path.join(restdir, restname)
            if not os.path.exists(restdir):
                raise TMError('restore', 'nodir', _('no such directory: {}'
                              ' -- cannot restore').format(restdir))
            os.rename(os.path.join(self.filedir, filename), restfile)
            os.remove(os.path.join(self.infodir, filename + '.trashinfo'))
            self.regenerate()
            self.log('restored {} to {}'.format(filename, restfile))
            if verbose:
                sys.stderr.write(_('restored {} to {}\n').format(filename,
                                 restfile))

        else:
            self.log('couldn\'t find {} in trash'.format(filename))
            raise TMError('restore', 'nofile', _('no such file in trash'))
